from allauth.account.models import EmailAddress
from django.contrib.auth.models import AbstractUser
from django.db import models
from django.utils.translation import gettext as _
from phonenumber_field.modelfields import PhoneNumberField

from lacrei_models.lacreiid.managers import UserManager
from lacrei_models.utils.models import (
    NULLABLE,
    BaseModel,
    HashedAutoField,
    HashedFileName,
)
from lacrei_models.utils.validators import OnlyAlphabeticValidator


class User(AbstractUser, BaseModel):
    USER = "user"
    PROFESSIONAL = "professional"
    LOGGED_AS_CHOICES = [(USER, _("Usuário")), (PROFESSIONAL, _("Profissional"))]
    id = HashedAutoField(primary_key=True)
    logged_as = models.CharField(
        max_length=12,
        choices=LOGGED_AS_CHOICES,
        null=True,
        blank=True,
        help_text=_("Indica se o usuário está logado como Usuário ou Profissional"),
    )
    email = models.EmailField(unique=True)
    first_name = models.CharField(
        max_length=150, blank=False, validators=[OnlyAlphabeticValidator()]
    )
    last_name = models.CharField(
        max_length=150, blank=False, validators=[OnlyAlphabeticValidator()]
    )
    birth_date = models.DateField(**NULLABLE)
    is_18_years_old_or_more = models.BooleanField(**NULLABLE)
    last_login = models.DateTimeField(auto_now_add=True)
    email_verified = models.BooleanField(default=False)

    accepted_privacy_document = models.BooleanField(default=False)
    privacy_document = models.ForeignKey(
        "lacreiid.PrivacyDocument", on_delete=models.PROTECT, null=True, blank=False
    )
    newsletter_subscribed = models.BooleanField(default=True)

    phone = PhoneNumberField(null=True, blank=True)
    phone_verified = models.BooleanField(default=False)
    phone_verification_token = models.CharField(max_length=6, **NULLABLE)
    phone_verification_token_expires_at = models.DateTimeField(**NULLABLE)

    objects = UserManager()
    username = None
    date_joined = None
    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = _("Pessoa Usuária")
        verbose_name_plural = _("Pessoas Usuárias")
        app_label = "lacreiid"

    def post_create_instance(self, *args, **kwargs):
        self.profile = Profile.objects.create(user=self)


class IntersectionalityBadge(models.Model):
    badge = models.ImageField(
        null=True,
        upload_to=HashedFileName("intersectionality_badges"),
        verbose_name=_("Selo de interseccionalidade"),
    )
    position_order = models.PositiveIntegerField(null=True, unique=True)

    class Meta:
        abstract = True


class DisabilityType(IntersectionalityBadge, BaseModel):
    name = models.CharField(max_length=100, blank=False)

    class Meta:
        verbose_name = _("Deficiência")
        verbose_name_plural = _("Deficiências")
        ordering = ["position_order"]
        app_label = "lacreiid"

    def __str__(self):
        return self.name


class SexualOrientation(IntersectionalityBadge, BaseModel):
    name = models.CharField(max_length=100, blank=False)

    class Meta:
        verbose_name = _("Orientação sexual")
        verbose_name_plural = _("Orientações sexuais")
        ordering = ["position_order"]
        app_label = "lacreiid"

    def __str__(self):
        return self.name


class GenderIdentity(IntersectionalityBadge, BaseModel):
    name = models.CharField(max_length=100, blank=False)

    class Meta:
        verbose_name = _("Identidade de gênero")
        verbose_name_plural = _("Identidades de gênero")
        ordering = ["position_order"]
        app_label = "lacreiid"

    def __str__(self):
        return self.name


class EthnicGroup(IntersectionalityBadge, BaseModel):
    name = models.CharField(max_length=100, blank=False)

    class Meta:
        verbose_name = _("Grupo étnico")
        verbose_name_plural = _("Grupos étnicos")
        ordering = ["position_order"]
        app_label = "lacreiid"

    def __str__(self):
        return self.name


class Pronoun(IntersectionalityBadge, BaseModel):
    article = models.CharField(max_length=1, blank=False)
    pronoun = models.CharField(max_length=50, blank=False)

    class Meta:
        verbose_name = _("Pronome")
        verbose_name_plural = _("Pronomes")
        ordering = ["position_order"]
        app_label = "lacreiid"

    def __str__(self):
        return f"{self.article}/{self.pronoun}"


class PrivacyDocument(BaseModel):
    privacy_policy = models.URLField()
    terms_of_use = models.URLField()
    profile_type = models.CharField(
        max_length=20,
        choices=[("lacreiid", _("Lacrei ID")), ("lacreisaude", _("Lacrei Saúde"))],
    )

    class Meta:
        verbose_name = _("Termo de uso e privacidade")
        verbose_name_plural = _("Termos de uso e privacidade")
        app_label = "lacreiid"

    def __str__(self):
        return f"{self.id} - {self.profile_type} - {self.created_at}"


class BaseProfile(BaseModel):
    ethnic_group = models.ForeignKey(
        EthnicGroup,
        on_delete=models.PROTECT,
        **NULLABLE,
        verbose_name=_("Grupo Étnico"),
    )
    gender_identity = models.ForeignKey(
        GenderIdentity,
        on_delete=models.PROTECT,
        **NULLABLE,
        verbose_name=_("Identidade de Gênero"),
    )
    sexual_orientation = models.ForeignKey(
        SexualOrientation,
        on_delete=models.PROTECT,
        **NULLABLE,
        verbose_name=_("Orientação Sexual"),
    )
    pronoun = models.ForeignKey(
        Pronoun, on_delete=models.PROTECT, **NULLABLE, verbose_name=_("Pronome")
    )
    disability_types = models.ManyToManyField(
        DisabilityType, blank=True, verbose_name=_("Tipos de deficiência")
    )

    other_ethnic_group = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outro Grupo Étnico")
    )
    other_gender_identity = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outra Identidade de Gênero")
    )
    other_sexual_orientation = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outra Orientação Sexual")
    )
    other_pronoun = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outro Pronome")
    )
    other_disability_types = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outro Tipos de deficiência")
    )
    other_article = models.CharField(
        max_length=1, **NULLABLE, verbose_name=_("Outro Artigo")
    )

    @property
    def display_ethnic_group(self):
        return self.other_ethnic_group or getattr(self.ethnic_group, "name", None)

    @property
    def display_gender_identity(self):
        return self.other_gender_identity or getattr(self.gender_identity, "name", None)

    @property
    def display_sexual_orientation(self):
        return self.other_sexual_orientation or getattr(
            self.sexual_orientation, "name", None
        )

    @property
    def display_pronoun(self):
        return self.other_pronoun or getattr(self.pronoun, "pronoun", None)

    @property
    def display_article(self):
        return self.other_article or getattr(self.pronoun, "article", None)

    @property
    def display_disability_types(self):
        disability_list = list(self.disability_types.values_list("name", flat=True))
        if self.other_disability_types:
            disability_list.append(self.other_disability_types)
        return ", ".join(disability_list) if disability_list else None

    class Meta:
        abstract = True


class Profile(BaseProfile):
    id = HashedAutoField(primary_key=True)
    user = models.OneToOneField(
        User, on_delete=models.PROTECT, verbose_name=_("Lacrei ID")
    )
    completed = models.BooleanField(
        default=False,
        verbose_name=_("Perfil completo"),
    )
    photo = models.ImageField(
        null=True,
        upload_to=HashedFileName("profile_photos"),
        verbose_name=_("Foto de perfil"),
    )
    photo_description = models.TextField(
        null=True, blank=True, help_text=_("Descrição da foto")
    )

    def __str__(self):
        try:
            return self.user.first_name
        except User.DoesNotExist:
            return "Perfil sem usuário"

    class Meta:
        verbose_name = _("Perfil")
        verbose_name_plural = _("Perfis")
        app_label = "lacreiid"


class CustomEmailAddress(EmailAddress):
    class Meta:
        proxy = True
        app_label = "lacreiid"



import logging

from django.utils.functional import cached_property
from django.utils.translation import gettext as _

PROFESSIONAL_VALIDATION_PENDING_ADMIN = _("Pendente Lacrei")
PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT = _("Pendente Profissional")
PROFESSIONAL_ALREADY_APPROVED_FOR_STEP = _("Aprovado")

logger = logging.getLogger(__name__)


class VerificationStep:
    name: str
    url_path_to_submit: str
    url_path_to_wait_completion: str
    description: str

    def __init__(self, professional):
        self.professional = professional

    @cached_property
    def is_completed(self):  # pragma: no cover
        raise NotImplementedError

    @cached_property
    def is_submitted(self):  # pragma: no cover
        raise NotImplementedError

    @cached_property
    def redirect_to(self):
        if self.is_completed:
            return None

        if self.is_submitted:
            return self.url_path_to_wait_completion

        return self.url_path_to_submit

    def to_representation(self):
        return {
            "is_completed": self.is_completed,
            "is_submitted": self.is_submitted,
            "redirect_to": self.redirect_to,
            "description": self.description,
            "internal_message": (
                PROFESSIONAL_ALREADY_APPROVED_FOR_STEP
                if self.is_completed
                else (
                    PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT
                    if not self.is_submitted
                    else PROFESSIONAL_VALIDATION_PENDING_ADMIN
                )
            ),
        }


class BoardRegistrationNumber(VerificationStep):
    """
    Uma pessoa interna da lacrei irá validar o número do conselho no respectivo portal."

    Após validação, será enviado um email de confirmação, onde:
    - Se rejeitado: apagamos o cadastro e pedimos para refazer, com um número certo.
    - Se aprovado: segue para o pós cadastro.
    """

    name = "board_registration_number"
    url_path_to_submit = "/"
    url_path_to_wait_completion = "/saude/verificacao-inscricao/"
    description = "1ª - Validação no conselho"

    @cached_property
    def is_completed(self):
        return self.professional.profile_status == "approved"

    @cached_property
    def is_submitted(self):
        return True


class EmailConfirmation(VerificationStep):
    """
    Nessa etapa, uma pessoa interna da Lacrei já aprovou o cadastro,
    e agora aguardamos a pessoa profissional confirmar o email.
    """

    name = "email_confirmation"
    url_path_to_submit = "/"
    url_path_to_wait_completion = "/saude/verificacao-inscricao/"
    description = "2ª - Confirmação do email"

    @cached_property
    def is_completed(self):
        return self.professional.user.email_verified

    @cached_property
    def is_submitted(self):
        # Se a etapa anterior (BoardRegistrationNumber) foi aprovada,
        # então esta etapa está automaticamente "submitted"
        return BoardRegistrationNumber(self.professional).is_completed


class PostRegistrationData(VerificationStep):
    """
    Nessa passo a pessoa profissional irá enviar os dados restantes para o cadastro.

    Após validação, será enviado um email de confirmação, onde:
    - Se rejeitado: mandamos email pedindo para reenviar os dados.
    - Se aprovado: Liberado para ficar ativo na plataforma
    """

    name = "post_registration_data"
    url_path_to_submit = "/saude/cadastro-dados-pessoais/"
    url_path_to_wait_completion = "/saude/painel-cadastro-analise/"
    description = "4ª - Pós cadastro"

    @cached_property
    def is_completed(self):
        return self.professional.active

    @cached_property
    def is_submitted(self):
        return hasattr(self.professional, "clinic") and bool(
            self.professional.board_certification_selfie
        )


class IntersectionalityData(VerificationStep):
    """
    Na etapa, pedimos que adicione os dados de interseccionalidade.
    """

    name = "add_intersectionality_data"
    url_path_to_submit = "/saude/cadastro-diversidade/"
    url_path_to_wait_completion = ""
    description = "3ª - Diversidade"

    @cached_property
    def is_completed(self):
        professional = self.professional
        intersectionality_fields = {
            "ethnic_group": bool(
                professional.ethnic_group_id or professional.other_ethnic_group
            ),
            "pronoun": bool(professional.pronoun_id or professional.other_pronoun),
            "sexual_orientation": bool(
                professional.sexual_orientation_id
                or professional.other_sexual_orientation
            ),
            "gender_identity": bool(
                professional.gender_identity_id or professional.other_gender_identity
            ),
        }
        return all(intersectionality_fields.values())

    @cached_property
    def is_submitted(self):
        return self.is_completed


class VerificationStepsService:
    def __init__(self, professional):
        self.professional = professional
        self.steps = [
            BoardRegistrationNumber(professional),
            EmailConfirmation(professional),
            IntersectionalityData(professional),
            PostRegistrationData(professional),
        ]

    def to_representation(self) -> dict:
        current_step = None
        steps_repr = {}

        for step in self.steps:
            if not step.is_completed and not current_step:
                current_step = step.name

            steps_repr[step.name] = step.to_representation()

        return {"current_step": current_step, "steps": steps_repr}

    def current_step(self):
        representation = self.to_representation()
        if not representation["current_step"]:
            return None
        return representation["steps"][representation["current_step"]]
